"""
Notifly Documentation

Discord: (object): Discord wrapper to send message, images to the channel using Webhooks
Slack (object): Slack wrapper to send message, images to the channel using API
Telegram (object): Telegram Web Client wrapper to send images, files over the bot using API
"""
